@@ -26,7 +26,6 @@
 #include <ctype.h>
 #include <unistd.h>
 
-#define MAINTAINER_OPTIONS
 
 #include "packet.h"
 #include "iobuf.h"
@@ -70,6 +69,7 @@ enum cmd_and_opt_values { aNull = 0,
     aKeygen,
     aSignEncr,
     aSignKey,
+    aLSignKey,
     aListPackets,
     aEditKey,
     aDeleteKey,
@@ -111,6 +111,9 @@ enum cmd_and_opt_values { aNull = 0,
     oKeyring,
     oSecretKeyring,
     oDefaultKey,
+    oDefRecipient,
+    oDefRecipientSelf,
+    oNoDefRecipient,
     oOptions,
     oDebug,
     oDebugAll,
@@ -169,6 +172,9 @@ enum cmd_and_opt_values { aNull = 0,
     oLoggerFD,
     oUtf8Strings,
     oNoUtf8Strings,
+    oDisableCipherAlgo,
+    oDisablePubkeyAlgo,
+    oAllowNonSelfsignedUID,
 aTest };
 
 
@@ -190,9 +196,11 @@ static ARGPARSE_OPTS opts[] = {
     { aCheckKeys, "check-sigs",256, N_("check key signatures")},
     { oFingerprint, "fingerprint", 256, N_("list keys and fingerprints")},
     { aListSecretKeys, "list-secret-keys", 256, N_("list secret keys")},
-    { aKeygen, "gen-key",   256, N_("generate a new key pair")},
+    { aKeygen,	   "gen-key",  256, N_("generate a new key pair")},
     { aDeleteKey, "delete-key",256, N_("remove key from the public keyring")},
-    { aEditKey, "edit-key"  ,256, N_("sign or edit a key")},
+    { aSignKey,  "sign-key"   ,256, N_("sign a key")},
+    { aLSignKey, "lsign-key"  ,256, N_("sign a key locally")},
+    { aEditKey,  "edit-key"   ,256, N_("sign or edit a key")},
     { aGenRevoke, "gen-revoke",256, N_("generate a revocation certificate")},
     { aExport, "export"           , 256, N_("export keys") },
     { aSendKeys, "send-keys"     , 256, N_("export keys to a key server") },
@@ -207,24 +215,26 @@ static ARGPARSE_OPTS opts[] = {
     { aImportOwnerTrust,
 	      "import-ownertrust", 256 , N_("import ownertrust values")},
     { aUpdateTrustDB,
-	      "update-trustdb",0 , N_("|[NAMES]|update the trust database")},
+	      "update-trustdb",0 , N_("update the trust database")},
     { aCheckTrustDB,
 	      "check-trustdb",0 , N_("|[NAMES]|check the trust database")},
     { aFixTrustDB, "fix-trustdb",0 , N_("fix a corrupted trust database")},
     { aDeArmor, "dearmor", 256, N_("De-Armor a file or stdin") },
     { aEnArmor, "enarmor", 256, N_("En-Armor a file or stdin") },
     { aPrintMD,  "print-md" , 256, N_("|algo [files]|print message digests")},
-    { aPrintMDs, "print-mds" , 256, N_("print all message digests")},
-    #ifdef MAINTAINER_OPTIONS
     { aPrimegen, "gen-prime" , 256, "@" },
     { aGenRandom, "gen-random" , 256, "@" },
-    #endif
 
     { 301, NULL, 0, N_("@\nOptions:\n ") },
 
     { oArmor, "armor",     0, N_("create ascii armored output")},
     { oRecipient, "recipient", 2, N_("|NAME|encrypt for NAME")},
     { oRecipient, "remote-user", 2, "@"},  /* old option name */
+    { oDefRecipient, "default-recipient" ,2,
+				  N_("|NAME|use NAME as default recipient")},
+    { oDefRecipientSelf, "default-recipient-self" ,0,
+				N_("use the default key as default recipient")},
+    { oNoDefRecipient, "no-default-recipient", 0, "@" },
     { oEncryptTo, "encrypt-to", 2, "@" },
     { oNoEncryptTo, "no-encrypt-to", 0, "@" },
     { oUser, "local-user",2, N_("use this user-id to sign or decrypt")},
@@ -279,11 +289,11 @@ static ARGPARSE_OPTS opts[] = {
 
   /* hidden options */
     { aExportOwnerTrust, "list-ownertrust",0 , "@"},  /* alias */
+    { aPrintMDs, "print-mds" , 256, "@"}, /* old */
     { aListTrustDB, "list-trustdb",0 , "@"},
     { aListTrustPath, "list-trust-path",0, "@"},
     { oKOption, NULL,	 0, "@"},
     { oPasswdFD, "passphrase-fd",1, "@" },
-    { aSignKey, "sign-key"  ,256, "@" }, /* alias for edit-key */
     { aDeleteSecretKey, "delete-secret-key",0, "@" },
     { oQuickRandom, "quick-random", 0, "@"},
     { oNoVerbose, "no-verbose", 0, "@"},
@@ -321,6 +331,9 @@ static ARGPARSE_OPTS opts[] = {
     { oUtf8Strings, "utf8-strings", 0, "@" },
     { oNoUtf8Strings, "no-utf8-strings", 0, "@" },
     { oWithFingerprint, "with-fingerprint", 0, "@" },
+    { oDisableCipherAlgo,  "disable-cipher-algo", 2, "@" },
+    { oDisablePubkeyAlgo,  "disable-pubkey-algo", 2, "@" },
+    { oAllowNonSelfsignedUID, "allow-non-selfsigned-uid", 0, "@" },
 {0} };
 
 
@@ -518,7 +531,8 @@ main( int argc, char **argv )
     int parse_debug = 0;
     int default_config =1;
     int default_keyring = 1;
-    int greeting = 1;
+    int greeting = 0;
+    int nogreeting = 0;
     enum cmd_and_opt_values cmd = 0;
     const char *trustdb_name = NULL;
     char *def_cipher_string = NULL;
@@ -652,25 +666,25 @@ main( int argc, char **argv )
 	  case aListKeys: set_cmd( &cmd, aListKeys); break;
 	  case aListSigs: set_cmd( &cmd, aListSigs); break;
 	  case aExportSecret: set_cmd( &cmd, aExportSecret); break;
-	  case aDeleteSecretKey: set_cmd( &cmd, aDeleteSecretKey); break;
-	  case aDeleteKey: set_cmd( &cmd, aDeleteKey); break;
+	  case aDeleteSecretKey: set_cmd( &cmd, aDeleteSecretKey);
+							greeting=1; break;
+	  case aDeleteKey: set_cmd( &cmd, aDeleteKey); greeting=1; break;
 
 	  case aDetachedSign: detached_sig = 1; set_cmd( &cmd, aSign ); break;
 	  case aSym: set_cmd( &cmd, aSym); break;
 	  case aDecrypt: set_cmd( &cmd, aDecrypt); break;
 	  case aEncr: set_cmd( &cmd, aEncr); break;
 	  case aSign: set_cmd( &cmd, aSign );  break;
-	  case aKeygen: set_cmd( &cmd, aKeygen); break;
+	  case aKeygen: set_cmd( &cmd, aKeygen); greeting=1; break;
 	  case aSignKey: set_cmd( &cmd, aSignKey); break;
+	  case aLSignKey: set_cmd( &cmd, aLSignKey); break;
 	  case aStore: set_cmd( &cmd, aStore); break;
-	  case aEditKey: set_cmd( &cmd, aEditKey); break;
+	  case aEditKey: set_cmd( &cmd, aEditKey); greeting=1; break;
 	  case aClearsign: set_cmd( &cmd, aClearsign); break;
 	  case aGenRevoke: set_cmd( &cmd, aGenRevoke); break;
 	  case aVerify: set_cmd( &cmd, aVerify); break;
-	#ifdef MAINTAINER_OPTIONS
 	  case aPrimegen: set_cmd( &cmd, aPrimegen); break;
 	  case aGenRandom: set_cmd( &cmd, aGenRandom); break;
-	#endif
 	  case aPrintMD: set_cmd( &cmd, aPrintMD); break;
 	  case aPrintMDs: set_cmd( &cmd, aPrintMDs); break;
 	  case aListTrustDB: set_cmd( &cmd, aListTrustDB); break;
@@ -678,8 +692,8 @@ main( int argc, char **argv )
 	  case aUpdateTrustDB: set_cmd( &cmd, aUpdateTrustDB); break;
 	  case aFixTrustDB: set_cmd( &cmd, aFixTrustDB); break;
 	  case aListTrustPath: set_cmd( &cmd, aListTrustPath); break;
-	  case aDeArmor: set_cmd( &cmd, aDeArmor); greeting = 0; break;
-	  case aEnArmor: set_cmd( &cmd, aEnArmor); greeting = 0; break;
+	  case aDeArmor: set_cmd( &cmd, aDeArmor); break;
+	  case aEnArmor: set_cmd( &cmd, aEnArmor); break;
 	  case aExportOwnerTrust: set_cmd( &cmd, aExportOwnerTrust); break;
 	  case aImportOwnerTrust: set_cmd( &cmd, aImportOwnerTrust); break;
 
@@ -715,7 +729,7 @@ main( int argc, char **argv )
 	    break;
 	  case oNoArmor: opt.no_armor=1; opt.armor=0; break;
 	  case oNoDefKeyring: default_keyring = 0; break;
-	  case oNoGreeting: greeting = 0; break;
+	  case oNoGreeting: nogreeting = 1; break;
 	  case oNoVerbose: g10_opt_verbose = 0;
 			   opt.verbose = 0; opt.list_sigs=0; break;
 	  case oQuickRandom: quick_random_gen(1); break;
@@ -727,6 +741,18 @@ main( int argc, char **argv )
 	  case oMaxCertDepth: opt.max_cert_depth = pargs.r.ret_int; break;
 	  case oTrustDBName: trustdb_name = pargs.r.ret_str; break;
 	  case oDefaultKey: opt.def_secret_key = pargs.r.ret_str; break;
+	  case oDefRecipient:
+		    if( *pargs.r.ret_str )
+			opt.def_recipient = make_username(pargs.r.ret_str);
+		    break;
+	  case oDefRecipientSelf:
+		    m_free(opt.def_recipient); opt.def_recipient = NULL;
+		    opt.def_recipient_self = 1;
+		    break;
+	  case oNoDefRecipient:
+		    m_free(opt.def_recipient); opt.def_recipient = NULL;
+		    opt.def_recipient_self = 0;
+		    break;
 	  case oNoOptions: break; /* no-options */
 	  case oHomedir: opt.homedir = pargs.r.ret_str; break;
 	  case oNoBatch: opt.batch = 0; break;
@@ -813,6 +839,15 @@ main( int argc, char **argv )
 	  case oNotation: add_notation_data( pargs.r.ret_str ); break;
 	  case oUtf8Strings: utf8_strings = 1; break;
 	  case oNoUtf8Strings: utf8_strings = 0; break;
+	  case oDisableCipherAlgo:
+		disable_cipher_algo( string_to_cipher_algo(pargs.r.ret_str) );
+		break;
+	  case oDisablePubkeyAlgo:
+		disable_pubkey_algo( string_to_pubkey_algo(pargs.r.ret_str) );
+		break;
+	  case oAllowNonSelfsignedUID:
+		opt.allow_non_selfsigned_uid = 1;
+		break;
 
 	  default : pargs.err = configfp? 1:2; break;
 	}
@@ -826,14 +861,22 @@ main( int argc, char **argv )
     m_free( configname ); configname = NULL;
     if( log_get_errorcount(0) )
 	g10_exit(2);
+    if( nogreeting )
+	greeting = 0;
 
     if( greeting ) {
 	fprintf(stderr, "%s %s; %s\n",
 			strusage(11), strusage(13), strusage(14) );
 	fprintf(stderr, "%s\n", strusage(15) );
-      #ifdef IS_DEVELOPMENT_VERSION
+    }
+  #ifdef IS_DEVELOPMENT_VERSION
+    if( !opt.batch )
 	log_info("NOTE: this is a development version!\n");
-      #endif
+  #endif
+    if( opt.force_mdc ) {
+	log_info("--force-mdc ignored because"
+		 " the OpenPGP WG has not yet aggreed on MDCs\n");
+	opt.force_mdc = 0;
     }
     if( opt.batch )
 	tty_batchmode( 1 );
@@ -1044,26 +1087,43 @@ main( int argc, char **argv )
 
 
       case aSignKey: /* sign the key given as argument */
+	if( argc != 1 )
+	    wrong_args(_("--sign-key user-id"));
+	username = make_username( fname );
+	keyedit_menu(fname, locusr, NULL, 1 );
+	m_free(username);
+	break;
+
+      case aLSignKey:
+	if( argc != 1 )
+	    wrong_args(_("--lsign-key user-id"));
+	username = make_username( fname );
+	keyedit_menu(fname, locusr, NULL, 2 );
+	m_free(username);
+	break;
+
       case aEditKey: /* Edit a key signature */
 	if( !argc )
-	    wrong_args(_("--edit-key username [commands]"));
+	    wrong_args(_("--edit-key user-id [commands]"));
+	username = make_username( fname );
 	if( argc > 1 ) {
 	    sl = NULL;
 	    for( argc--, argv++ ; argc; argc--, argv++ )
-		append_to_strlist2( &sl, *argv, utf8_strings );
-	    keyedit_menu( fname, locusr, sl );
+		append_to_strlist( &sl, *argv );
+	    keyedit_menu( username, locusr, sl, 0 );
 	    free_strlist(sl);
 	}
 	else
-	    keyedit_menu(fname, locusr, NULL );
+	    keyedit_menu(username, locusr, NULL, 0 );
+	m_free(username);
 	break;
 
       case aDeleteSecretKey:
 	if( argc != 1 )
-	    wrong_args(_("--delete-secret-key username"));
+	    wrong_args(_("--delete-secret-key user-id"));
       case aDeleteKey:
 	if( argc != 1 )
-	    wrong_args(_("--delete-key username"));
+	    wrong_args(_("--delete-key user-id"));
 	username = make_username( fname );
 	if( (rc = delete_key(username, cmd==aDeleteSecretKey)) )
 	    log_error("%s: delete key failed: %s\n", username, g10_errstr(rc) );
@@ -1171,68 +1231,70 @@ main( int argc, char **argv )
 	break;
 
 
-     #ifdef MAINTAINER_OPTIONS
       case aPrimegen:
-	if( argc == 1 ) {
-	    mpi_print( stdout, generate_public_prime( atoi(argv[0]) ), 1);
+	{   int mode = argc < 2 ? 0 : atoi(*argv);
+
+	    if( mode == 1 && argc == 2 ) {
+		mpi_print( stdout, generate_public_prime( atoi(argv[1]) ), 1);
+	    }
+	    else if( mode == 2 && argc == 3 ) {
+		mpi_print( stdout, generate_elg_prime(
+					     0, atoi(argv[1]),
+					     atoi(argv[2]), NULL,NULL ), 1);
+	    }
+	    else if( mode == 3 && argc == 3 ) {
+		MPI *factors;
+		mpi_print( stdout, generate_elg_prime(
+					     1, atoi(argv[1]),
+					     atoi(argv[2]), NULL,&factors ), 1);
+		putchar('\n');
+		mpi_print( stdout, factors[0], 1 ); /* print q */
+	    }
+	    else if( mode == 4 && argc == 3 ) {
+		MPI g = mpi_alloc(1);
+		mpi_print( stdout, generate_elg_prime(
+						 0, atoi(argv[1]),
+						 atoi(argv[2]), g, NULL ), 1);
+		putchar('\n');
+		mpi_print( stdout, g, 1 );
+		mpi_free(g);
+	    }
+	    else
+		wrong_args("--gen-prime mode bits [qbits] ");
 	    putchar('\n');
 	}
-	else if( argc == 2 ) {
-	    mpi_print( stdout, generate_elg_prime( 0, atoi(argv[0]),
-						   atoi(argv[1]), NULL,NULL ), 1);
-	    putchar('\n');
-	}
-	else if( argc == 3 ) {
-	    MPI g = mpi_alloc(1);
-	    mpi_print( stdout, generate_elg_prime( 0, atoi(argv[0]),
-						   atoi(argv[1]), g, NULL ), 1);
-	    printf("\nGenerator: ");
-	    mpi_print( stdout, g, 1 );
-	    putchar('\n');
-	    mpi_free(g);
-	}
-	else if( argc == 4 ) {
-	    mpi_print( stdout, generate_elg_prime( 1, atoi(argv[0]),
-						   atoi(argv[1]), NULL,NULL ), 1);
-	    putchar('\n');
-	}
-	else
-	    usage(1);
 	break;
-      #endif /* MAINTAINER OPTIONS */
 
-      #ifdef MAINTAINER_OPTIONS
       case aGenRandom:
-	if( argc < 1 || argc > 2 )
-	    wrong_args("--gen-random level [hex]");
 	{
-	    int c;
-	    int level = atoi(*argv);
-	    for(;;) {
+	    int level = argc ? atoi(*argv):0;
+	    int count = argc > 1 ? atoi(argv[1]): 0;
+	    int endless = !count;
+
+	    if( argc < 1 || argc > 2 || level < 0 || level > 2 || count < 0 )
+		wrong_args("--gen-random 0|1|2 [count]");
+
+	    while( endless || count ) {
 		byte *p;
-		if( argc == 2 ) {
-		    p = get_random_bits( 8, level, 0);
-		    printf("%02x", *p );
-		    fflush(stdout);
-		}
-		else {
-		    p = get_random_bits( 800, level, 0);
-		    for(c=0; c < 100; c++ )
-			putchar( p[c] );
-		}
+		size_t n = !endless && count < 100? count : 100;
+
+		p = get_random_bits( n*8, level, 0);
+		fwrite( p, n, 1, stdout );
 		m_free(p);
+		if( !endless )
+		    count -= n;
 	    }
 	}
 	break;
-      #endif /* MAINTAINER OPTIONS */
 
       case aPrintMD:
 	if( argc < 1)
-	    wrong_args("--print-md algo [file]");
-	else {
-	    int algo = string_to_digest_algo(*argv);
+	    wrong_args("--print-md algo [files]");
+	{
+	    int all_algos = (**argv=='*' && !(*argv)[1]);
+	    int algo = all_algos? 0 : string_to_digest_algo(*argv);
 
-	    if( !algo )
+	    if( !algo && !all_algos )
 		log_error(_("invalid hash algorithm `%s'\n"), *argv );
 	    else {
 		argc--; argv++;
@@ -1246,7 +1308,7 @@ main( int argc, char **argv )
 	}
 	break;
 
-      case aPrintMDs:
+      case aPrintMDs: /* old option */
 	if( !argc )
 	    print_mds(NULL,0);
 	else {
@@ -1290,7 +1352,7 @@ main( int argc, char **argv )
 
       case aListTrustPath:
 	if( !argc )
-	    wrong_args("--list-trust-path <usernames>");
+	    wrong_args("--list-trust-path <user-ids>");
 	for( ; argc; argc--, argv++ ) {
 	    username = make_username( *argv );
 	    list_trust_path( username );
