@@ -34,6 +34,7 @@
 #include "trustdb.h"
 #include "main.h"
 #include "i18n.h"
+#include "status.h"
 
 
 static struct {
@@ -58,6 +59,7 @@ static int import_secret_one( const char *fname, KBNODE keyblock );
 static int import_revoke_cert( const char *fname, KBNODE node );
 static int chk_self_sigs( const char *fname, KBNODE keyblock,
 			  PKT_public_key *pk, u32 *keyid );
+static void mark_non_selfsigned_uids_valid( KBNODE keyblock, u32 *kid );
 static int delete_inv_parts( const char *fname, KBNODE keyblock, u32 *keyid );
 static int merge_blocks( const char *fname, KBNODE keyblock_orig,
 			 KBNODE keyblock, u32 *keyid,
@@ -199,6 +201,24 @@ import( IOBUF inp, int fast, const char* fname )
 	    log_info(_(" secret keys unchanged: %lu\n"), stats.secret_dups );
     }
 
+    if( is_status_enabled() ) {
+	char buf[12*16];
+	sprintf(buf, "%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu",
+		count,
+		stats.no_user_id,
+		stats.imported,
+		stats.imported_rsa,
+		stats.unchanged,
+		stats.n_uids,
+		stats.n_subk,
+		stats.n_sigs,
+		stats.n_revoc,
+		stats.secret_read,
+		stats.secret_imported,
+		stats.secret_dups);
+	write_status_text( STATUS_IMPORT_RES, buf );
+    }
+
     return rc;
 }
 
@@ -348,6 +368,9 @@ import_one( const char *fname, KBNODE keyblock, int fast )
     if( rc )
 	return rc== -1? 0:rc;
 
+    if( opt.allow_non_selfsigned_uid )
+	mark_non_selfsigned_uids_valid( keyblock, keyid );
+
     if( !delete_inv_parts( fname, keyblock, keyid ) ) {
 	if( !opt.quiet ) {
 	    log_info( _("key %08lX: no valid user ids\n"),
@@ -385,6 +408,11 @@ import_one( const char *fname, KBNODE keyblock, int fast )
 	/* we are ready */
 	if( !opt.quiet )
 	    log_info( _("key %08lX: public key imported\n"), (ulong)keyid[1]);
+	if( is_status_enabled() ) {
+	    char *us = get_long_user_id_string( keyid );
+	    write_status_text( STATUS_IMPORTED, us );
+	    m_free(us);
+	}
 	stats.imported++;
 	if( is_RSA( pk->pubkey_algo ) )
 	    stats.imported_rsa++;
@@ -662,7 +690,7 @@ import_revoke_cert( const char *fname, KBNODE node )
  * loop over the keyblock and check all self signatures.
  * Mark all user-ids with a self-signature by setting flag bit 0.
  * Mark all user-ids with an invalid self-signature by setting bit 1.
- * This works allso for subkeys, here the subkey is marked.
+ * This works also for subkeys, here the subkey is marked.
  */
 static int
 chk_self_sigs( const char *fname, KBNODE keyblock,
@@ -702,7 +730,7 @@ chk_self_sigs( const char *fname, KBNODE keyblock,
 		    knode = find_prev_kbnode( keyblock,
 						 n, PKT_SECRET_SUBKEY );
 
-		if( !knode )  {
+		if( !knode ) {
 		    log_error( _("key %08lX: no subkey for key binding\n"),
 					    (ulong)keyid[1]);
 		    n->flag |= 4; /* delete this */
@@ -725,6 +753,29 @@ chk_self_sigs( const char *fname, KBNODE keyblock,
     return 0;
 }
 
+
+
+/****************
+ * If a user ID has at least one signature, mark it as valid
+ */
+static void
+mark_non_selfsigned_uids_valid( KBNODE keyblock, u32 *kid )
+{
+    KBNODE node;
+    for(node=keyblock->next; node; node = node->next ) {
+	if( node->pkt->pkttype == PKT_USER_ID && !(node->flag & 1) ) {
+	    if( node->next && node->next->pkt->pkttype == PKT_SIGNATURE ) {
+		node->flag |= 1;
+		log_info( _("key %08lX: accepted non self-signed userid '"),
+							 (ulong)kid[1]);
+		print_string( log_stream(), node->pkt->pkt.user_id->name,
+					    node->pkt->pkt.user_id->len, 0 );
+		fputs("'\n", log_stream() );
+	    }
+	}
+    }
+}
+
 /****************
  * delete all parts which are invalid and those signatures whose
  * public key algorithm is not available in this implemenation;
