@@ -50,6 +50,8 @@ struct {
     int def_digest_algo;
     int def_compress_algo;
     const char *def_secret_key;
+    char *def_recipient;
+    int def_recipient_self;
     int no_comment;
     int no_version;
     int marginals_needed;
@@ -78,6 +80,7 @@ struct {
     STRLIST notation_data;
     const char *set_policy_url;
     int use_embedded_filename;
+    int allow_non_selfsigned_uid;
 } opt;
 
 
