@@ -60,12 +60,12 @@ typedef struct {
 /*-- logger.c --*/
 void log_set_logfile( const char *name, int fd );
 FILE *log_stream(void);
+void g10_log_print_prefix(const char *text);
 void log_set_name( const char *name );
 const char *log_get_name(void);
 void log_set_pid( int pid );
 int  log_get_errorcount( int clear );
 void g10_log_hexdump( const char *text, const char *buf, size_t len );
-void g10_log_mpidump( const char *text, MPI a );
 
 #if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 5 )
   void g10_log_bug( const char *fmt, ... )
@@ -100,7 +100,6 @@ void g10_log_mpidump( const char *text, MPI a );
 #endif
 
 #define log_hexdump g10_log_hexdump
-#define log_mpidump g10_log_mpidump
 #define log_bug     g10_log_bug
 #define log_bug0    g10_log_bug0
 #define log_fatal   g10_log_fatal
@@ -148,11 +147,13 @@ const char *print_fname_stdout( const char *s );
 
 /*-- miscutil.c --*/
 u32 make_timestamp(void);
+u32 scan_isodatestr( const char *string );
 u32 add_days_to_timestamp( u32 stamp, u16 days );
 const char *strtimevalue( u32 stamp );
 const char *strtimestamp( u32 stamp ); /* GMT */
 const char *asctimestamp( u32 stamp ); /* localized */
 void print_string( FILE *fp, const byte *p, size_t n, int delim );
+char *make_printable_string( const byte *p, size_t n, int delim );
 int answer_is_yes( const char *s );
 int answer_is_yes_no_quit( const char *s );
 
@@ -160,7 +161,9 @@ int answer_is_yes_no_quit( const char *s );
 void free_strlist( STRLIST sl );
 #define FREE_STRLIST(a) do { free_strlist((a)); (a) = NULL ; } while(0)
 STRLIST add_to_strlist( STRLIST *list, const char *string );
+STRLIST add_to_strlist2( STRLIST *list, const char *string, int is_utf8 );
 STRLIST append_to_strlist( STRLIST *list, const char *string );
+STRLIST append_to_strlist2( STRLIST *list, const char *string, int is_utf8 );
 STRLIST strlist_prev( STRLIST head, STRLIST node );
 STRLIST strlist_last( STRLIST node );
 const char *memistr( const char *buf, size_t buflen, const char *sub );
@@ -175,8 +178,6 @@ char *native_to_utf8( const char *string );
 char *utf8_to_native( const char *string );
 int  check_utf8_string( const char *string );
 
-#define stricmp(a,b) strcasecmp((a),(b))
-
 #ifndef HAVE_MEMICMP
 int memicmp( const char *a, const char *b, size_t n );
 #endif
@@ -192,7 +193,9 @@ char *strlwr(char *a);
 #ifndef HAVE_MEMMOVE
   #define memmove(d, s, n) bcopy((s), (d), (n))
 #endif
-
+#ifndef HAVE_STRICMP
+  #define stricmp(a,b)	 strcasecmp( (a), (b) )
+#endif
 
 /**** other missing stuff ****/
 #ifndef HAVE_ATEXIT  /* For SunOS */
